@@ -32,6 +32,7 @@
 #include "got_lib_delta.h"
 #include "got_lib_inflate.h"
 #include "got_lib_object.h"
+#include "got_lib_hash.h"
 #include "got_lib_object_parse.h"
 
 #ifndef nitems
@@ -234,13 +235,14 @@ static const struct got_error got_errors[] = {
 	{ GOT_ERR_BAD_REQUEST, "unexpected request received" },
 	{ GOT_ERR_CLIENT_ID, "unknown client identifier" },
 	{ GOT_ERR_REPO_TEMPFILE, "no repository tempfile available" },
-	{ GOT_ERR_REFS_PROTECTED, "reference namespace may not be modified" },
-	{ GOT_ERR_REF_PROTECTED," reference may not be modified" },
+	{ GOT_ERR_REFS_PROTECTED, "reference namespace is protected" },
+	{ GOT_ERR_REF_PROTECTED, "reference is protected" },
 	{ GOT_ERR_REF_BUSY, "reference cannot be updated; please try again" },
 	{ GOT_ERR_COMMIT_BAD_AUTHOR, "commit author formatting would "
 	    "make Git unhappy" },
 	{ GOT_ERR_UID, "bad user ID" },
 	{ GOT_ERR_GID, "bad group ID" },
+	{ GOT_ERR_NO_PROG, "command not found or not accessible" },
 };
 
 static struct got_custom_error {
