@@ -46,6 +46,7 @@
 #include "got_lib_inflate.h"
 #include "got_lib_object.h"
 #include "got_lib_object_parse.h"
+#include "got_lib_object_qid.h"
 #include "got_lib_object_cache.h"
 #include "got_lib_pack.h"
 #include "got_lib_repository.h"
@@ -54,58 +55,31 @@
 #define nitems(_a) (sizeof(_a) / sizeof((_a)[0]))
 #endif
 
-struct got_object_id *
-got_object_id_dup(struct got_object_id *id1)
-{
-	struct got_object_id *id2;
-
-	id2 = malloc(sizeof(*id2));
-	if (id2 == NULL)
-		return NULL;
-	memcpy(id2, id1, sizeof(*id2));
-	return id2;
-}
-
-int
-got_object_id_cmp(const struct got_object_id *id1,
-    const struct got_object_id *id2)
-{
-	return memcmp(id1->sha1, id2->sha1, SHA1_DIGEST_LENGTH);
-}
-
 const struct got_error *
-got_object_qid_alloc_partial(struct got_object_qid **qid)
+got_object_type_label(const char **label, int obj_type)
 {
-	*qid = malloc(sizeof(**qid));
-	if (*qid == NULL)
-		return got_error_from_errno("malloc");
+	const struct got_error *err = NULL;
 
-	(*qid)->data = NULL;
-	return NULL;
-}
-
-const struct got_error *
-got_object_id_str(char **outbuf, struct got_object_id *id)
-{
-	static const size_t len = GOT_OBJECT_ID_HEX_MAXLEN;
-
-	*outbuf = malloc(len);
-	if (*outbuf == NULL)
-		return got_error_from_errno("malloc");
-
-	if (got_object_id_hex(id, *outbuf, len) == NULL) {
-		free(*outbuf);
-		*outbuf = NULL;
-		return got_error(GOT_ERR_BAD_OBJ_ID_STR);
+	switch (obj_type) {
+	case GOT_OBJ_TYPE_BLOB:
+		*label = GOT_OBJ_LABEL_BLOB;
+		break;
+	case GOT_OBJ_TYPE_TREE:
+		*label = GOT_OBJ_LABEL_TREE;
+		break;
+	case GOT_OBJ_TYPE_COMMIT:
+		*label = GOT_OBJ_LABEL_COMMIT;
+		break;
+	case GOT_OBJ_TYPE_TAG:
+		*label = GOT_OBJ_LABEL_TAG;
+		break;
+	default:
+		*label = NULL;
+		err = got_error(GOT_ERR_OBJ_TYPE);
+		break;
 	}
 
-	return NULL;
-}
-
-char *
-got_object_id_hex(struct got_object_id *id, char *buf, size_t len)
-{
-	return got_sha1_digest_to_str(id->sha1, buf, len);
+	return err;
 }
 
 void
@@ -158,24 +132,6 @@ got_object_raw_close(struct got_raw_object *obj)
 	return err;
 }
 
-void
-got_object_qid_free(struct got_object_qid *qid)
-{
-	free(qid);
-}
-
-void
-got_object_id_queue_free(struct got_object_id_queue *ids)
-{
-	struct got_object_qid *qid;
-
-	while (!STAILQ_EMPTY(ids)) {
-		qid = STAILQ_FIRST(ids);
-		STAILQ_REMOVE_HEAD(ids, entry);
-		got_object_qid_free(qid);
-	}
-}
-
 const struct got_error *
 got_object_parse_header(struct got_object **obj, char *buf, size_t len)
 {
@@ -1194,52 +1150,3 @@ done:
 		got_object_close(obj);
 	return err;
 }
-
-const struct got_error *
-got_read_file_to_mem(uint8_t **outbuf, size_t *outlen, FILE *f)
-{
-	const struct got_error *err = NULL;
-	static const size_t blocksize = 512;
-	size_t n, total, remain;
-	uint8_t *buf;
-
-	*outbuf = NULL;
-	*outlen = 0;
-
-	buf = malloc(blocksize);
-	if (buf == NULL)
-		return got_error_from_errno("malloc");
-
-	remain = blocksize;
-	total = 0;
-	for (;;) {
-		if (remain == 0) {
-			uint8_t *newbuf;
-			newbuf = reallocarray(buf, 1, total + blocksize);
-			if (newbuf == NULL) {
-				err = got_error_from_errno("reallocarray");
-				goto done;
-			}
-			buf = newbuf;
-			remain += blocksize;
-		}
-		n = fread(buf + total, 1, remain, f);
-		if (n == 0) {
-			if (ferror(f)) {
-				err = got_ferror(f, GOT_ERR_IO);
-				goto done;
-			}
-			break; /* EOF */
-		}
-		remain -= n;
-		total += n;
-	};
-
-done:
-	if (err == NULL) {
-		*outbuf = buf;
-		*outlen = total;
-	} else
-		free(buf);
-	return err;
-}
