@@ -42,6 +42,7 @@
 #include "got_lib_delta_cache.h"
 #include "got_lib_inflate.h"
 #include "got_lib_object.h"
+#include "got_lib_object_qid.h"
 #include "got_lib_object_parse.h"
 #include "got_lib_privsep.h"
 #include "got_lib_pack.h"
@@ -325,8 +326,7 @@ checksum:
 		got_hash_update(&ctx, h->trailer->packfile_sha1,
 		    SHA1_DIGEST_LENGTH);
 		got_hash_final(&ctx, hash);
-		if (got_hash_cmp(ctx.algo, hash, h->trailer->packidx_sha1)
-		    != 0)
+		if (got_hash_cmp(ctx.algo, hash, h->trailer->packidx_sha1) != 0)
 			err = got_error(GOT_ERR_PACKIDX_CSUM);
 	}
 done:
